!**************************************************************
! July 12 2001         
! Combines the 4 engineering mode data files into a file
! containing a combined single image
! 2005-06-01 VNY A small bug provoking crash on full-frame low-resolution
! images is fixed (SSC-SPR-3461). This was caused by insufficient length
! of the string-variable for a warning message
!***************************************************************
module omcomb_mod

    use param
    use oal
    use caloaldefs
    use dal

    use sasdal
    use errorHandling
     
     
   implicit none
  
   public omcomb
       integer(kind=int32), dimension(: , :), allocatable :: combinedImage  ! Stores the combined image
 
contains

!---------------------------------------------------------------------
subroutine omcomb
!---------------------------------------------------------------------
    integer :: noFiles
    !character(len=100) :: string     
    character(len=255) :: string          
    !************************
    ! Read in parameters.
    !************************

    noFiles = parameterCount("imagesets")

    !write(*,*) "L35 noFiles=", noFiles

    !*************************************
    ! Has to be 3 or 4 files
    !*************************************
    if (noFiles .lt. 3 .or. noFiles .gt. 4) then
        write(string, '(a, i2, a)') "Only ", noFiles, " were provided"
        call fatal("4 image files are required", trim(string))
    end if 
    !*************************************
    ! Create the combined image
    !*************************************
    !write(*,*) "L47"
    call  CreateCombinedImage(noFiles)
 
    !********************************************
    ! Create the output file
    !********************************************
    call CreateOutputFile


    !************************************
    ! Release heap memory
    !************************************
    deallocate(combinedImage) 

end subroutine omcomb
!****************************************************
! Create Output File 
!****************************************************
subroutine CreateOutputFile
    type(DataSetT) :: inDataSetH                                       ! An input dataset handle.
   
    type(DataSetT) :: outDataSetH                                      ! Output dataset handle.
    type(ArrayT) :: inArrayH, outArrayH                                          ! Output array handle.
    integer(kind=int32), dimension(:, :), pointer :: outarrayP               ! Integer array pointer
    integer(kind=int32), dimension(2) :: imageDim                          ! Image dimension.
 

    inDataSetH = dataSet(trim(stringParameter("imagesets", 0)) , read)
    inArrayH = array(inDataSetH, "OMIMI1", read)
        
    outDataSetH = dataSet(trim(stringParameter("outset")), create)
  
    !*****************************************************
    ! Add the image  array to the file
    !*****************************************************
    imageDim(1) = 1024
    imageDim(2) = 1024
    outArrayH = addArray(outDataSetH, "OMIMI1", integer32, imageDim)
   
  
    outArrayP => int32Array2Data(outArrayH)
    outArrayP = combinedImage


 
    !****************************************************
    !  Add attributes.
    !****************************************************

    !write(*,*) "L96"

    call copyAllAttributes(attributable(inArrayH), attributable(outArrayH))
 
    !***********************************************************************
    ! NB These values are fixed atv the values they should be and not what
    ! may be in the original IMI file headers - because they may not be 
    !***********************************************************************

    call setAttribute(outArrayH, "BINBPE",   "T", "BPE binning enabled")
    call setAttribute(outArrayH, "BINAX1",   1, " ", "CCD x-axis binning 2**N")
    call setAttribute(outArrayH, "BINAX2",   1, " ", "CCD y-axis binning 2**M")
    call setAttribute(outArrayH, "WINDOWX0", 0, " ", "Bottom left x co-ordinate of window")
    call setAttribute(outArrayH, "WINDOWY0", 0, " ", "Bottom left y co-ordinate of window")
    call setAttribute(outArrayH, "WINDOWDX", 2048, " ", "X dimension of window (centroided pixels")
    call setAttribute(outArrayH, "WINDOWDY", 2048, " ", "Y dimension of window (centroided pixels")
    call setAttribute(outArrayH, "SCIWIN", 1, " ", "SCIENCE WINDOW?")
   
    call release(inArrayH)
    call release(inDataSetH)

    call release(outArrayH)
    call release(outDataSetH)
end subroutine CreateOutputFile
!*************************************
! Subroutine CreateCombinedImage
!*************************************
subroutine CreateCombinedImage(noFiles)
    integer, intent(in) :: noFiles
    integer, dimension(1 : 4) :: windowX0
    integer :: windowY0, windowDX, windowDY
    character(len=255) :: set(4)                                      ! OM OSW Source List filename.
    type(DataSetT) :: inDataSetH                                      ! Input dataset handle.
    type(ArrayT) :: inArrayH
    integer(kind=int32), dimension(: , :), pointer :: imageStripP
    integer i, j, k, l
    logical binBPE
    integer :: x0    
    real(kind=single), dimension(: , :), pointer :: inOswp   
    integer :: dim(2)
    !character(len=100) :: string
    character(len=255) :: string
   
    !**********************************************************
    ! Allocate memory for the array to hold the combined image
    !**********************************************************
    !write(*,*) "L141"

    allocate(combinedImage(0 : 1023, 0 : 1023))
    combinedImage = -1.0

    !write(*,*) "L146"

    do i = 1, noFiles

        set(i) = trim(stringParameter("imagesets", i - 1))
        inDataSetH = dataSet(set(i) , read)

        !write(*,*) "L152 i=", i

       
        if (.not. hasBlock(inDataSetH, "OMIMI1")) then  ! Incompatible file
            write(string, '(a, a)') "Could not find any compatible blocks in set", set(i)
            call fatal("badFileType", string)
        end if

          
        inArrayH = array(inDataSetH, "OMIMI1", read)
        dim = dimensions(inArrayH)

        !**************************************
        ! Check the image dimensions - has
        ! to be 256 x 1024 
        !**************************************

        if (dim(1) .ne. 256) then
            call fatal("Invalid image x-dimension", set(i))
        end if

        if (dim(2) .ne. 1024) then
            call fatal("Invalid image x-dimension", set(i))
        end if

        !write(*,*) "L178"

        windowX0(i) = int32Attribute(inArrayH, "WINDOWX0")  
        windowY0 = int32Attribute(inArrayH, "WINDOWY0")
        windowDx = int32Attribute(inArrayH, "WINDOWDX")
        windowDy = int32Attribute(inArrayH, "WINDOWDY")
    
        binBPE = booleanAttribute(inArrayH, "BINBPE")
        !write(*,*) "L186 binBPE=", binBPE

        if (.not. binBPE) then
           !write(*,*) "L189"
!###############################################  VNY 31.05.2005 ###############
! The string variable is too short for the message (the file name might be quite long
            !write(string, '(a24, a)') "Invalid BINBPE for file ", trim(set(i))
           !write(string, '(a, a)') "Invalid BINBPE for file ", trim(set(i))
            !call warning(string, "BINBPE should be TRUE")
            windowX0(i) = 2 * windowX0(i)  
            windowY0 = 2 * windowY0
            windowDx = 2 * windowDx
            windowDy = 2 * windowDy
       
        end if
        !"write(*,*) "L199"      
        if (windowDy .ne. 2048) then
            write(string, '(a, i5, a, a)') "Invalid WINDOWDY (", windowDy, ") for file ", trim(set(i))
            call warning(string, "WINDOWDY should be 2048")
        end if                                

        !write(*,*) "L203"
        !************************************
        ! Check window x-origins
        !************************************ 
        do j = 1, i - 1
            if (windowX0(j) .eq. windowX0(i)) then
                write(string, FMT='(a, a, a, a, a)') "Files ", trim(set(i)), &
                                   " and ", trim(set(j)), "  have the same WINDOWX0"
                call fatal("bad window origins", string)
            end if
     
        end do

        do j = 0, noFiles
            if (j .eq. noFiles) then
                write(string, '(a, i6, a, a)') "Invalid WINDOWX0 (", windowX0(j + 1), ") for file ", trim(set(i))
                call fatal(string, "WINDOWXO should be one of 0, 512, 1024 or 1536")
            end if 

            if (windowX0(j + 1) .eq. j * 512) exit
        end do 
        !**********************************************
        ! Add this image segment to the combined image
        !**********************************************
 
        imageStripP => int32Array2Data(inArrayH)
        x0 = windowX0(i) / 2

        forall (k  = 0 : 255)
            forall (l = 0 : 1023) combinedImage(x0 + k, l) = imageStripP(k, l)
        end forall
    
    
   
        call release(inArrayH)
        call release(inDataSetH)


    end do
end subroutine CreateCombinedImage
            
end module  omcomb_mod











